@@ -24,7 +24,8 @@
 #include "zm_monitor.h"
 
 extern "C" {
-#include "libavutil/time.h"
+#include <libavutil/time.h>
+#include <libavutil/display.h>
 }
 
 #include <string>
@@ -50,6 +51,7 @@ VideoStore::CodecData VideoStore::codec_data[] = {
   { AV_CODEC_ID_H264, "h264", "h264", AV_PIX_FMT_YUV420P, AV_PIX_FMT_YUV420P,  AV_HWDEVICE_TYPE_NONE },
   { AV_CODEC_ID_H264, "h264", "libx264", AV_PIX_FMT_YUV420P, AV_PIX_FMT_YUV420P, AV_HWDEVICE_TYPE_NONE  },
   { AV_CODEC_ID_MJPEG, "mjpeg", "mjpeg", AV_PIX_FMT_YUVJ422P, AV_PIX_FMT_YUVJ422P, AV_HWDEVICE_TYPE_NONE },
+  { AV_CODEC_ID_AV1, "av1", "libaom-av1", AV_PIX_FMT_YUV420P, AV_PIX_FMT_YUV420P, AV_HWDEVICE_TYPE_NONE },
 #else
   { AV_CODEC_ID_H265, "h265", "libx265", AV_PIX_FMT_YUV420P, AV_PIX_FMT_YUV420P },
 
@@ -112,7 +114,7 @@ VideoStore::VideoStore(
 {
   FFMPEGInit();
   swscale.init();
-  opkt = new AVPacket;
+  opkt = av_packet_ptr{av_packet_alloc()};
 }  // VideoStore::VideoStore
 
 bool VideoStore::open() {
@@ -150,8 +152,12 @@ bool VideoStore::open() {
   } else {
     const AVDictionaryEntry *entry = av_dict_get(opts, "reorder_queue_size", nullptr, AV_DICT_MATCH_CASE);
     if (entry) {
-      reorder_queue_size = std::stoul(entry->value);
-      Debug(1, "reorder_queue_size set to %zu", reorder_queue_size);
+      if (monitor->GetOptVideoWriter() == Monitor::ENCODE) {
+        Debug(1, "reorder_queue_size ignored for non-passthrough");
+      } else {
+        reorder_queue_size = std::stoul(entry->value);
+        Debug(1, "reorder_queue_size set to %zu", reorder_queue_size);
+      }
       // remove it to prevent complaining later.
       av_dict_set(&opts, "reorder_queue_size", nullptr, AV_DICT_MATCH_CASE);
     }
@@ -367,12 +373,39 @@ bool VideoStore::open() {
 
   reorder_queues[video_out_stream->index] = {};
 
-  video_out_stream->time_base = video_in_stream ? video_in_stream->time_base : AV_TIME_BASE_Q;
   if (monitor->GetOptVideoWriter() == Monitor::PASSTHROUGH) {
+    video_out_stream->time_base = video_in_stream ? video_in_stream->time_base : AV_TIME_BASE_Q;
     // Only set orientation if doing passthrough, otherwise the frame image will be rotated
     Monitor::Orientation orientation = monitor->getOrientation();
-    if (orientation) {
+    if (orientation > 1) { // 1 is ROTATE_0
+#if LIBAVCODEC_VERSION_CHECK(59, 37, 100, 37, 100)
+      int32_t* displaymatrix = static_cast<int32_t*>(av_malloc(sizeof(int32_t)*9));
       Debug(3, "Have orientation %d", orientation);
+      if (orientation == Monitor::ROTATE_0) {
+      } else if (orientation == Monitor::ROTATE_90) {
+        av_display_rotation_set(displaymatrix, 90);
+      } else if (orientation == Monitor::ROTATE_180) {
+        av_display_rotation_set(displaymatrix, 180);
+      } else if (orientation == Monitor::ROTATE_270) {
+        av_display_rotation_set(displaymatrix, 270);
+      } else {
+        Warning("Unsupported Orientation(%d)", orientation);
+      }
+#endif
+#if LIBAVCODEC_VERSION_CHECK(60, 31, 102, 31, 102)
+      av_packet_side_data_add(
+          &video_out_stream->codecpar->coded_side_data,
+          &video_out_stream->codecpar->nb_coded_side_data,
+          AV_PKT_DATA_DISPLAYMATRIX,
+          (int32_t *)displaymatrix, sizeof(int32_t)*9, 0);
+#else
+#if LIBAVCODEC_VERSION_CHECK(59, 37, 100, 37, 100)
+      av_stream_add_side_data(video_out_stream,
+          AV_PKT_DATA_DISPLAYMATRIX,
+          (uint8_t *)displaymatrix,
+          sizeof(*displaymatrix));
+#endif
+#endif
       if (orientation == Monitor::ROTATE_0) {
       } else if (orientation == Monitor::ROTATE_90) {
         ret = av_dict_set(&video_out_stream->metadata, "rotate", "90", 0);
@@ -386,7 +419,10 @@ bool VideoStore::open() {
       } else {
         Warning("Unsupported Orientation(%d)", orientation);
       }
-    } // end if orientation
+    } // end if orientation > ROTATE_0
+  } else {
+    // encode
+    video_out_stream->time_base = AV_TIME_BASE_Q;
   }  // end if passthrough
 
   if (audio_in_stream and audio_in_ctx) {
@@ -469,13 +505,6 @@ bool VideoStore::open() {
       }  // end if
       audio_out_ctx->codec_tag = 0;
 #endif
-
-      if (audio_out_ctx->channels > 1) {
-        Warning("Audio isn't mono, changing it.");
-        audio_out_ctx->channels = 1;
-      } else {
-        Debug(3, "Audio is mono");
-      }
     } // end if is AAC
 
     if (oc->oformat->flags & AVFMT_GLOBALHEADER) {
@@ -515,6 +544,9 @@ bool VideoStore::open() {
     if (ret < 0) {
       Warning("Could not parse ffmpeg output options '%s'", options.c_str());
     }
+    if (reorder_queue_size) {
+      av_dict_set(&opts, "reorder_queue_size", nullptr, AV_DICT_MATCH_CASE);
+    }
   }
 
   const AVDictionaryEntry *movflags_entry = av_dict_get(opts, "movflags", nullptr, AV_DICT_MATCH_CASE);
@@ -554,11 +586,12 @@ bool VideoStore::open() {
 void VideoStore::flush_codecs() {
   // The codec queues data.  We need to send a flush command and out
   // whatever we get. Failures are not fatal.
-  AVPacket pkt;
-  // Without these we seg fault becuse av_init_packet doesn't init them
-  pkt.data = nullptr;
-  pkt.size = 0;
-  av_init_packet(&pkt);
+  av_packet_ptr pkt{av_packet_alloc()};
+
+  if (!pkt) {
+    Error("Unable to allocate packet.");
+    return;
+  }
 
   // I got crashes if the codec didn't do DELAY, so let's test for it.
   if (video_out_ctx->codec && ( video_out_ctx->codec->capabilities & 
@@ -569,12 +602,29 @@ void VideoStore::flush_codecs() {
 #endif
         )) {
     // Put encoder into flushing mode
-    while ((zm_send_frame_receive_packet(video_out_ctx, nullptr, pkt)) > 0) {
-      av_packet_rescale_ts(&pkt,
-          video_out_ctx->time_base,
-          video_out_stream->time_base);
-      write_packet(&pkt, video_out_stream);
-      zm_av_packet_unref(&pkt);
+    while (0 > zm_send_frame_receive_packet(video_out_ctx, nullptr, pkt.get())) {
+      av_packet_guard pkt_guard{pkt};
+      av_packet_rescale_ts(pkt.get(), video_out_ctx->time_base, video_out_stream->time_base);
+      if (pkt->duration <= 0) {
+        if (video_last_pts != AV_NOPTS_VALUE) {
+          pkt->duration = pkt->pts - video_last_pts;
+#if LIBAVCODEC_VERSION_CHECK(57, 64, 0, 64, 0)
+          Debug(1, "duration calc: pts(%" PRId64 ") - last_pts(%" PRId64 ") = (%" PRId64 ") => (%" PRId64 ")",
+              pkt->pts,
+              video_last_pts,
+              pkt->pts - video_last_pts,
+              pkt->duration
+              );
+#endif
+          if (pkt->duration <= 0) {
+            pkt->duration = av_rescale_q(1, video_in_stream->time_base, video_out_stream->time_base);
+          }
+        } else {
+          pkt->duration = av_rescale_q(1, video_in_stream->time_base, video_out_stream->time_base);
+        }
+      }
+      video_last_pts = pkt->pts;
+      write_packet(pkt.get(), video_out_stream);
     } // while have buffered frames
     Debug(1, "Done writing buffered video.");
   } // end if have delay capability
@@ -593,12 +643,12 @@ void VideoStore::flush_codecs() {
       if (zm_add_samples_to_fifo(fifo, out_frame)) {
         // Should probably set the frame size to what is reported FIXME
         if (zm_get_samples_from_fifo(fifo, out_frame)) {
-          if (zm_send_frame_receive_packet(audio_out_ctx, out_frame, pkt) > 0) {
-            av_packet_rescale_ts(&pkt,
+          if (zm_send_frame_receive_packet(audio_out_ctx, out_frame, pkt.get()) > 0) {
+            av_packet_guard pkt_guard{pkt};
+            av_packet_rescale_ts(pkt.get(),
                 audio_out_ctx->time_base,
                 audio_out_stream->time_base);
-            write_packet(&pkt, audio_out_stream);
-            zm_av_packet_unref(&pkt);
+            write_packet(pkt.get(), audio_out_stream);
           }
         }  // end if data returned from fifo
       }
@@ -614,14 +664,14 @@ void VideoStore::flush_codecs() {
 
       // SHould probably set the frame size to what is reported FIXME
       if (av_audio_fifo_read(fifo, (void **)out_frame->data, frame_size)) {
-        if (zm_send_frame_receive_packet(audio_out_ctx, out_frame, pkt)) {
-          pkt.stream_index = audio_out_stream->index;
+        if (zm_send_frame_receive_packet(audio_out_ctx, out_frame, pkt.get())) {
+          av_packet_guard pkt_guard{pkt};
+          pkt->stream_index = audio_out_stream->index;
 
-          av_packet_rescale_ts(&pkt,
+          av_packet_rescale_ts(pkt.get(),
               audio_out_ctx->time_base,
               audio_out_stream->time_base);
-          write_packet(&pkt, audio_out_stream);
-          zm_av_packet_unref(&pkt);
+          write_packet(pkt.get(), audio_out_stream);
         }
       }  // end if data returned from fifo
     }  // end while still data in the fifo
@@ -631,17 +681,17 @@ void VideoStore::flush_codecs() {
       avcodec_send_frame(audio_out_ctx, nullptr);
 #endif
 
-    while (1) {
-      if (0 >= zm_receive_packet(audio_out_ctx, pkt)) {
+    while (true) {
+      if (0 >= zm_receive_packet(audio_out_ctx, *pkt)) {
         Debug(1, "No more packets");
         break;
       }
+      av_packet_guard pkt_guard{pkt};
 
       ZM_DUMP_PACKET(pkt, "raw from encoder");
-      av_packet_rescale_ts(&pkt, audio_out_ctx->time_base, audio_out_stream->time_base);
+      av_packet_rescale_ts(pkt.get(), audio_out_ctx->time_base, audio_out_stream->time_base);
       ZM_DUMP_STREAM_PACKET(audio_out_stream, pkt, "writing flushed packet");
-      write_packet(&pkt, audio_out_stream);
-      zm_av_packet_unref(&pkt);
+      write_packet(pkt.get(), audio_out_stream);
     }  // while have buffered frames
   }  // end if audio_out_codec
 }  // end flush_codecs
@@ -802,16 +852,30 @@ bool VideoStore::setup_resampler() {
   Debug(2, "Got something other than AAC (%s)", audio_in_codec->name);
 
   // Some formats (i.e. WAV) do not produce the proper channel layout
+#if LIBAVCODEC_VERSION_CHECK(59, 24, 100, 24, 100)
+  // channel_layout is deprecated need to use ch_layout
+  // based on changes to ffmpeg/doc/examples/transcoding.c
+  if (audio_in_ctx->ch_layout.order == AV_CHANNEL_ORDER_UNSPEC) {
+    char buf[64];
+    av_channel_layout_default(&audio_in_ctx->ch_layout, audio_in_ctx->ch_layout.nb_channels);
+    ret = av_channel_layout_describe(&audio_in_ctx->ch_layout, buf, sizeof(buf));
+    Info( "Setting input channel layout fron unspecified to default ret=%d describe=%s)", ret, buf);
+  }
+#else
   if (audio_in_ctx->channel_layout == 0) {
     Debug(2, "Setting input channel layout to mono");
     // Perhaps we should not be modifying the audio_in_ctx....
     audio_in_ctx->channel_layout = av_get_channel_layout("mono");
   }
+#endif
 
   /* put sample parameters */
   audio_out_ctx->bit_rate = audio_in_ctx->bit_rate <= 32768 ? audio_in_ctx->bit_rate : 32768;
   audio_out_ctx->sample_rate = audio_in_ctx->sample_rate;
   audio_out_ctx->sample_fmt = audio_in_ctx->sample_fmt;
+#if LIBAVCODEC_VERSION_CHECK(59, 24, 100, 24, 0)
+  av_channel_layout_copy(&audio_out_ctx->ch_layout, &audio_in_ctx->ch_layout);
+#else
   audio_out_ctx->channels = audio_in_ctx->channels;
   audio_out_ctx->channel_layout = audio_in_ctx->channel_layout;
   audio_out_ctx->sample_fmt = audio_in_ctx->sample_fmt;
@@ -824,6 +888,8 @@ bool VideoStore::setup_resampler() {
       audio_out_ctx->channel_layout = av_get_default_channel_layout(audio_out_ctx->channels);
   }
 #endif
+#endif
+
   if (audio_out_codec->supported_samplerates) {
     int found = 0;
     for (unsigned int i = 0; audio_out_codec->supported_samplerates[i]; i++) {
@@ -889,22 +955,37 @@ bool VideoStore::setup_resampler() {
         audio_out_ctx->time_base.num, audio_out_ctx->time_base.den);
 
   Debug(1,
-        "Audio in bit_rate (%" AV_PACKET_DURATION_FMT ") sample_rate(%d) channels(%d) fmt(%d) layout(%" PRIi64 ") frame_size(%d)",
+        "Audio in bit_rate (%" AV_PACKET_DURATION_FMT ") sample_rate(%d) channels(%d) fmt(%d) frame_size(%d)",
         audio_in_ctx->bit_rate, audio_in_ctx->sample_rate,
-        audio_in_ctx->channels, audio_in_ctx->sample_fmt,
-        audio_in_ctx->channel_layout, audio_in_ctx->frame_size);
+#if LIBAVCODEC_VERSION_CHECK(59, 24, 100, 24, 0)
+        audio_in_ctx->ch_layout.nb_channels,
+#else
+        audio_in_ctx->channels,
+#endif
+        audio_in_ctx->sample_fmt,
+        audio_in_ctx->frame_size);
   Debug(1,
-        "Audio out context bit_rate (%" AV_PACKET_DURATION_FMT ") sample_rate(%d) channels(%d) fmt(%d) layout(% " PRIi64 ") frame_size(%d)",
+        "Audio out context bit_rate (%" AV_PACKET_DURATION_FMT ") sample_rate(%d) channels(%d) fmt(%d) frame_size(%d)",
         audio_out_ctx->bit_rate, audio_out_ctx->sample_rate,
-        audio_out_ctx->channels, audio_out_ctx->sample_fmt,
-        audio_out_ctx->channel_layout, audio_out_ctx->frame_size);
+#if LIBAVCODEC_VERSION_CHECK(59, 24, 100, 24, 0)
+        audio_out_ctx->ch_layout.nb_channels,
+#else
+        audio_out_ctx->channels,
+#endif
+        audio_out_ctx->sample_fmt,
+        audio_out_ctx->frame_size);
 
 #if LIBAVCODEC_VERSION_CHECK(57, 64, 0, 64, 0)
   Debug(1,
-        "Audio out stream bit_rate (%" PRIi64 ") sample_rate(%d) channels(%d) fmt(%d) layout(%" PRIi64 ") frame_size(%d)",
+        "Audio out stream bit_rate (%" PRIi64 ") sample_rate(%d) channels(%d) fmt(%d) frame_size(%d)",
         audio_out_stream->codecpar->bit_rate, audio_out_stream->codecpar->sample_rate,
-        audio_out_stream->codecpar->channels, audio_out_stream->codecpar->format,
-        audio_out_stream->codecpar->channel_layout, audio_out_stream->codecpar->frame_size);
+#if LIBAVCODEC_VERSION_CHECK(59, 24, 100, 24, 0)
+        audio_out_stream->codecpar->ch_layout.nb_channels,
+#else
+        audio_out_stream->codecpar->channels,
+#endif
+        audio_out_stream->codecpar->format,
+        audio_out_stream->codecpar->frame_size);
 #else
   Debug(1,
       "Audio out bit_rate (%d) sample_rate(%d) channels(%d) fmt(%d) "
@@ -932,11 +1013,29 @@ bool VideoStore::setup_resampler() {
 
   if (!(fifo = av_audio_fifo_alloc(
           audio_out_ctx->sample_fmt,
-          audio_out_ctx->channels, 1))) {
+#if LIBAVCODEC_VERSION_CHECK(59, 24, 100, 24, 0)
+          audio_out_ctx->ch_layout.nb_channels
+#else
+          audio_out_ctx->channels
+#endif
+          , 1))) {
     Error("Could not allocate FIFO");
     return false;
   }
 #if defined(HAVE_LIBSWRESAMPLE)
+#if LIBAVCODEC_VERSION_CHECK(59, 24, 100, 24, 100)
+  if ((ret = swr_alloc_set_opts2(&resample_ctx,
+      &audio_out_ctx->ch_layout,
+      audio_out_ctx->sample_fmt,
+      audio_out_ctx->sample_rate,
+      &audio_in_ctx->ch_layout,
+      audio_in_ctx->sample_fmt,
+      audio_in_ctx->sample_rate,
+      0, nullptr)) < 0) {
+    Error("Could not allocate resample context");
+    return false;
+  }
+#else
   resample_ctx = swr_alloc_set_opts(nullptr,
       audio_out_ctx->channel_layout,
       audio_out_ctx->sample_fmt,
@@ -951,6 +1050,7 @@ bool VideoStore::setup_resampler() {
     av_frame_free(&out_frame);
     return false;
   }
+#endif
   if ((ret = swr_init(resample_ctx)) < 0) {
     Error("Could not open resampler");
     av_frame_free(&in_frame);
@@ -1000,15 +1100,24 @@ bool VideoStore::setup_resampler() {
   out_frame->nb_samples = audio_out_ctx->frame_size;
   out_frame->format = audio_out_ctx->sample_fmt;
 #if LIBAVCODEC_VERSION_CHECK(56, 8, 0, 60, 100)
+#if LIBAVCODEC_VERSION_CHECK(59, 24, 100, 24, 100)
+  out_frame->ch_layout = audio_out_ctx->ch_layout,
+#else
   out_frame->channels = audio_out_ctx->channels;
-#endif
   out_frame->channel_layout = audio_out_ctx->channel_layout;
+#endif
+#endif
   out_frame->sample_rate = audio_out_ctx->sample_rate;
 
   // The codec gives us the frame size, in samples, we calculate the size of the
   // samples buffer in bytes
   unsigned int audioSampleBuffer_size = av_samples_get_buffer_size(
-      nullptr, audio_out_ctx->channels,
+      nullptr,
+#if LIBAVCODEC_VERSION_CHECK(59, 24, 100, 24, 100)
+      audio_out_ctx->ch_layout.nb_channels,
+#else
+      audio_out_ctx->channels,
+#endif
       audio_out_ctx->frame_size,
       audio_out_ctx->sample_fmt, 0);
   converted_in_samples = reinterpret_cast<uint8_t *>(av_malloc(audioSampleBuffer_size));
@@ -1022,7 +1131,12 @@ bool VideoStore::setup_resampler() {
 
   // Setup the data pointers in the AVFrame
   if (avcodec_fill_audio_frame(
-        out_frame, audio_out_ctx->channels,
+        out_frame,
+#if LIBAVCODEC_VERSION_CHECK(59, 24, 100, 24, 100)
+        audio_out_ctx->ch_layout.nb_channels,
+#else
+        audio_out_ctx->channels,
+#endif
         audio_out_ctx->sample_fmt,
         (const uint8_t *)converted_in_samples,
         audioSampleBuffer_size, 0) < 0) {
@@ -1034,11 +1148,16 @@ bool VideoStore::setup_resampler() {
 #endif
 }  // end bool VideoStore::setup_resampler()
 
-int VideoStore::writePacket(const std::shared_ptr<ZMPacket> &zm_pkt) {
+int VideoStore::writePacket(const std::shared_ptr<ZMPacket> zm_pkt) {
   int stream_index;
   if (zm_pkt->codec_type == AVMEDIA_TYPE_VIDEO) {
     stream_index = video_out_stream->index;
   } else if (zm_pkt->codec_type == AVMEDIA_TYPE_AUDIO) {
+    if (!audio_out_stream) {
+      Debug(1, "Called writeAudioFramePacket when no audio_out_stream");
+      return 0;
+      // FIXME -ve return codes do not free packet in ffmpeg_camera at the moment
+    }
     stream_index = audio_out_stream->index;
   } else {
     Error("Unknown stream type in packet (%d)", zm_pkt->codec_type);
@@ -1047,13 +1166,13 @@ int VideoStore::writePacket(const std::shared_ptr<ZMPacket> &zm_pkt) {
   auto &queue = reorder_queues[stream_index];
   Debug(1, "Queue size for %d is %zu", stream_index, queue.size());
 
-  AVPacket *av_pkt = &zm_pkt->packet;
+  AVPacket *av_pkt = zm_pkt->packet.get();
   // queue the packet
   bool have_out_of_order = false;
   auto rit = queue.rbegin();
   // Find the previous packet for the stream, and check dts
   while (rit != queue.rend()) {
-    AVPacket *p = &((*rit)->packet);
+    AVPacket *p = ((*rit)->packet).get();
     if (p->dts <= av_pkt->dts) {
       Debug(1, "Found in order packet");
       // packets are in order, everything is fine
@@ -1088,7 +1207,9 @@ int VideoStore::writePacket(const std::shared_ptr<ZMPacket> &zm_pkt) {
   return 0;
 }
 
-int VideoStore::writeVideoFramePacket(const std::shared_ptr<ZMPacket> &zm_packet) {
+int VideoStore::writeVideoFramePacket(const std::shared_ptr<ZMPacket> zm_packet) {
+  av_packet_guard pkt_guard;
+
   frame_count += 1;
 
   // if we have to transcode
@@ -1122,7 +1243,7 @@ int VideoStore::writeVideoFramePacket(const std::shared_ptr<ZMPacket> &zm_packet
             );
       } else if (!zm_packet->in_frame) {
         Debug(4, "Have no in_frame");
-        if (zm_packet->packet.size and !zm_packet->decoded) {
+        if (zm_packet->packet->size and !zm_packet->decoded) {
           Debug(4, "Decoding");
           if (!zm_packet->decode(video_in_ctx)) {
             Debug(2, "unable to decode yet.");
@@ -1176,11 +1297,13 @@ int VideoStore::writeVideoFramePacket(const std::shared_ptr<ZMPacket> &zm_packet
     // Do this to allow the encoder to choose whether to use I/P/B frame
     //zm_packet->out_frame->pict_type = AV_PICTURE_TYPE_NONE;
     //zm_packet->out_frame->key_frame = zm_packet->keyframe;
-#if LIBAVCODEC_VERSION_CHECK(57, 64, 0, 64, 0)
+#if LIBAVCODEC_VERSION_CHECK(60, 3, 0, 3, 0)
+    frame->duration = 0;
+#elif LIBAVCODEC_VERSION_CHECK(57, 64, 0, 64, 0)
     frame->pkt_duration = 0;
 #endif
 
-    int64_t in_pts = zm_packet->timestamp.tv_sec * (uint64_t)1000000 + zm_packet->timestamp.tv_usec;
+    int64_t in_pts = (zm_packet->timestamp.tv_sec * (uint64_t)1000000) + zm_packet->timestamp.tv_usec;
     if (video_first_pts == AV_NOPTS_VALUE) {
       video_first_pts = in_pts;
       Debug(2, "No video_first_pts, set to (%" PRId64 ") secs(%" PRIi64 ") usecs(%" PRIi64 ")",
@@ -1191,52 +1314,64 @@ int VideoStore::writeVideoFramePacket(const std::shared_ptr<ZMPacket> &zm_packet
     } else {
       uint64_t useconds = in_pts - video_first_pts;
       frame->pts = av_rescale_q(useconds, AV_TIME_BASE_Q, video_out_ctx->time_base);
-      Debug(2,
-            "Setting pts for frame(%d) to (%" PRId64 ") from (start %" PRIu64 " - %" PRIu64 " - secs(%" PRIi64 ") usecs(%" PRIi64 ") @ %d/%d",
+      Debug(2, 
+            "Setting pts for frame(%d) to (%" PRId64 ") from (start %" PRIu64 " - %" PRIu64 " - secs(%" PRIi64 ".%" PRIi64 ") @ %d/%d",
             frame_count,
             frame->pts,
             video_first_pts,
-            useconds,
+            in_pts,
             static_cast<int64>(zm_packet->timestamp.tv_sec),
             static_cast<int64>(zm_packet->timestamp.tv_usec),
             video_out_ctx->time_base.num,
             video_out_ctx->time_base.den);
     }
 
-    av_init_packet(opkt);
-    opkt->data = nullptr;
-    opkt->size = 0;
-
-    int ret = zm_send_frame_receive_packet(video_out_ctx, frame, *opkt);
+    int ret = zm_send_frame_receive_packet(video_out_ctx, frame, opkt.get());
     if (ret <= 0) {
       if (ret < 0) {
         Error("Could not send frame (error '%s')", av_make_error_string(ret).c_str());
       }
       return ret;
     }
-    ZM_DUMP_PACKET((*opkt), "packet returned by codec");
+    pkt_guard.acquire(opkt);
+    ZM_DUMP_PACKET(opkt, "packet returned by codec");
 
     // Need to adjust pts/dts values from codec time to stream time
     if (opkt->pts != AV_NOPTS_VALUE)
       opkt->pts = av_rescale_q(opkt->pts, video_out_ctx->time_base, video_out_stream->time_base);
     if (opkt->dts != AV_NOPTS_VALUE)
       opkt->dts = av_rescale_q(opkt->dts, video_out_ctx->time_base, video_out_stream->time_base);
-    Debug(1, "Timebase conversions using %d/%d -> %d/%d",
+    Debug(2, "Timebase conversions using %d/%d -> %d/%d",
         video_out_ctx->time_base.num,
         video_out_ctx->time_base.den,
         video_out_stream->time_base.num,
         video_out_stream->time_base.den);
 
+#if 0
     int64_t duration = 0;
     if (zm_packet->in_frame) {
-      if (zm_packet->in_frame->pkt_duration) {
+
+      if (
+#if LIBAVCODEC_VERSION_CHECK(60, 3, 0, 3, 0)
+          zm_packet->in_frame->duration
+#else
+          zm_packet->in_frame->pkt_duration
+#endif
+         ) {
         duration = av_rescale_q(
+#if LIBAVCODEC_VERSION_CHECK(60, 3, 0, 3, 0)
+            zm_packet->in_frame->duration,
+#else
             zm_packet->in_frame->pkt_duration,
+#endif
             video_in_stream->time_base,
             video_out_stream->time_base);
-        Debug(1, "duration from ipkt: pts(%" PRId64 ") = pkt_duration(%" PRId64 ") => (%" PRId64 ") (%d/%d) (%d/%d)",
-            zm_packet->in_frame->pts,
+        Warning( "duration from ipkt: = duration(%" PRId64 ") => (%" PRId64 ") (%d/%d) (%d/%d)",
+#if LIBAVCODEC_VERSION_CHECK(60, 3, 0, 3, 0)
+            zm_packet->in_frame->duration,
+#else
             zm_packet->in_frame->pkt_duration,
+#endif
             duration,
             video_in_stream->time_base.num,
             video_in_stream->time_base.den,
@@ -1248,73 +1383,83 @@ int VideoStore::writeVideoFramePacket(const std::shared_ptr<ZMPacket> &zm_packet
               zm_packet->in_frame->pts - video_last_pts,
               video_in_stream->time_base,
               video_out_stream->time_base);
-        Debug(1, "duration calc: pts(%" PRId64 ") - last_pts(%" PRId64 ") = (%" PRId64 ") => (%" PRId64 ")",
+        Warning( "duration calc: pts(%" PRId64 ") - last_pts(%" PRId64 ") = (%" PRId64 ") => (%" PRId64 ")",
             zm_packet->in_frame->pts,
             video_last_pts,
             zm_packet->in_frame->pts - video_last_pts,
             duration
             );
         if (duration <= 0) {
+#if LIBAVCODEC_VERSION_CHECK(60, 3, 0, 3, 0)
+          duration = zm_packet->in_frame->duration ?
+            zm_packet->in_frame->duration :
+            av_rescale_q(1, video_in_stream->time_base, video_out_stream->time_base);
+#else
           duration = zm_packet->in_frame->pkt_duration ?
             zm_packet->in_frame->pkt_duration :
             av_rescale_q(1, video_in_stream->time_base, video_out_stream->time_base);
+#endif
         }
       }  // end if in_frmae->pkt_duration
       video_last_pts = zm_packet->in_frame->pts;
     } else {
+
       //duration = av_rescale_q(zm_packet->out_frame->pts - video_last_pts, video_in_stream->time_base, video_out_stream->time_base);
     }  // end if in_frmae
-    opkt->duration = duration;
+#endif
+    
+    if (video_last_pts != AV_NOPTS_VALUE) {
+      opkt->duration = opkt->pts - video_last_pts;
+#if LIBAVCODEC_VERSION_CHECK(57, 64, 0, 64, 0)
+      Debug(2, "duration calc: pts %" PRId64 " - last_pts %" PRId64 " = %" PRId64 ,
+          opkt->pts,
+          video_last_pts,
+          opkt->duration
+          );
+    } else {
+      Debug(2, "Setting duration to %" PRId64 " dts is %" PRId64 " pts is %" PRId64, opkt->duration, opkt->dts, opkt->pts);
+#endif
+    }
+
+    video_last_pts = opkt->pts;
+    Debug(2, "Setting video_last_pts to %" PRId64 " because it's the first packet", video_last_pts);
   } else { // Passthrough
-    AVPacket *ipkt = &zm_packet->packet;
-    ZM_DUMP_STREAM_PACKET(video_in_stream, (*ipkt), "Doing passthrough, just copy packet");
+    AVPacket *ipkt = zm_packet->packet.get();
+    ZM_DUMP_STREAM_PACKET(video_in_stream, ipkt, "Doing passthrough, just copy packet");
     // Just copy it because the codec is the same
-    av_init_packet(opkt);
-    opkt->data = ipkt->data;
-    opkt->size = ipkt->size;
-    opkt->flags = ipkt->flags;
-    opkt->duration = ipkt->duration;
-    av_packet_ref(opkt, ipkt);
+    av_packet_ref(opkt.get(), ipkt);
+    pkt_guard.acquire(opkt);
 
     if (ipkt->dts != AV_NOPTS_VALUE) {
       if (video_first_dts == AV_NOPTS_VALUE) {
-        Debug(2, "Starting video first_dts will become %" PRId64, ipkt->dts);
+        Debug(1, "Starting video first_dts will become %" PRId64, ipkt->dts);
         video_first_dts = ipkt->dts;
       }
       opkt->dts = ipkt->dts - video_first_dts;
-    //} else {
-      //opkt.dts = next_dts[video_out_stream->index] ? av_rescale_q(next_dts[video_out_stream->index], video_out_stream->time_base, video_in_stream->time_base) : 0;
-      //Debug(3, "Setting dts to video_next_dts %" PRId64 " from %" PRId64, opkt.dts, next_dts[video_out_stream->index]);
     }
-    if ((ipkt->pts != AV_NOPTS_VALUE) and (video_first_dts != AV_NOPTS_VALUE)) {
+    if (ipkt->pts != AV_NOPTS_VALUE) {
       opkt->pts = ipkt->pts - video_first_dts;
     }
-
-    av_packet_rescale_ts(opkt, video_in_stream->time_base, video_out_stream->time_base);
+    if ((ipkt->pts != AV_NOPTS_VALUE) and (ipkt->dts != AV_NOPTS_VALUE)) {
+      av_packet_rescale_ts(opkt.get(), video_in_stream->time_base, video_out_stream->time_base);
+    }
   }  // end if codec matches
 
-  write_packet(opkt, video_out_stream);
-  zm_av_packet_unref(opkt);
+  write_packet(opkt.get(), video_out_stream);
   if (hw_frame) av_frame_free(&hw_frame);
 
   return 1;
 }  // end int VideoStore::writeVideoFramePacket( AVPacket *ipkt )
 
-int VideoStore::writeAudioFramePacket(const std::shared_ptr<ZMPacket> &zm_packet) {
-  if (!audio_out_stream) {
-    Debug(1, "Called writeAudioFramePacket when no audio_out_stream");
-    return 0;
-    // FIXME -ve return codes do not free packet in ffmpeg_camera at the moment
-  }
-
-  AVPacket *ipkt = &zm_packet->packet;
+int VideoStore::writeAudioFramePacket(const std::shared_ptr<ZMPacket> zm_packet) {
+  AVPacket *ipkt = zm_packet->packet.get();
   int ret;
-  ZM_DUMP_STREAM_PACKET(audio_in_stream, (*ipkt), "input packet");
+  ZM_DUMP_STREAM_PACKET(audio_in_stream, ipkt, "input packet");
 
   if (audio_first_dts == AV_NOPTS_VALUE) {
     audio_first_dts = ipkt->dts;
     audio_next_pts = audio_out_ctx->frame_size;
-    Debug(3, "audio first_dts to %" PRId64, audio_first_dts);
+    Debug(1, "audio first_dts to %" PRId64, audio_first_dts);
   }
 
   // Need to adjust pts before feeding to decoder.... should really copy the pkt instead of modifying it
@@ -1346,17 +1491,16 @@ int VideoStore::writeAudioFramePacket(const std::shared_ptr<ZMPacket> &zm_packet
 
       zm_dump_frame(out_frame, "Out frame after resample");
 
-      av_init_packet(opkt);
-      if (zm_send_frame_receive_packet(audio_out_ctx, out_frame, *opkt) <= 0)
+      if (zm_send_frame_receive_packet(audio_out_ctx, out_frame, opkt.get()) <= 0)
         break;
 
       // Scale the PTS of the outgoing packet to be the correct time base
-      av_packet_rescale_ts(opkt,
+      av_packet_rescale_ts(opkt.get(),
           audio_out_ctx->time_base,
           audio_out_stream->time_base);
 
-      write_packet(opkt, audio_out_stream);
-      zm_av_packet_unref(opkt);
+      write_packet(opkt.get(), audio_out_stream);
+      zm_av_packet_unref(opkt.get());
 
       if (zm_resample_get_delay(resample_ctx, out_frame->sample_rate) < out_frame->nb_samples)
         break;
@@ -1364,7 +1508,6 @@ int VideoStore::writeAudioFramePacket(const std::shared_ptr<ZMPacket> &zm_packet
       input_frame = nullptr;
     }  // end while there is data in the resampler
   } else {
-    av_init_packet(opkt);
     opkt->data = ipkt->data;
     opkt->size = ipkt->size;
     opkt->flags = ipkt->flags;
@@ -1377,12 +1520,12 @@ int VideoStore::writeAudioFramePacket(const std::shared_ptr<ZMPacket> &zm_packet
       opkt->dts = ipkt->dts;
     }
 
-    ZM_DUMP_STREAM_PACKET(audio_in_stream, (*ipkt), "after pts adjustment");
-    av_packet_rescale_ts(opkt, audio_in_stream->time_base, audio_out_stream->time_base);
-    ZM_DUMP_STREAM_PACKET(audio_out_stream, (*opkt), "after stream pts adjustment");
-    write_packet(opkt, audio_out_stream);
+    ZM_DUMP_STREAM_PACKET(audio_in_stream, ipkt, "after pts adjustment");
+    av_packet_rescale_ts(opkt.get(), audio_in_stream->time_base, audio_out_stream->time_base);
+    ZM_DUMP_STREAM_PACKET(audio_out_stream, opkt, "after stream pts adjustment");
+    write_packet(opkt.get(), audio_out_stream);
 
-    zm_av_packet_unref(opkt);
+    zm_av_packet_unref(opkt.get());
   }  // end if encoding or copying
 
   return 0;
@@ -1394,17 +1537,28 @@ int VideoStore::write_packet(AVPacket *pkt, AVStream *stream) {
 
   if (pkt->dts == AV_NOPTS_VALUE) {
     if (last_dts[stream->index] == AV_NOPTS_VALUE) {
-      last_dts[stream->index] = 0;
+      last_dts[stream->index] = -1;
     } 
     pkt->dts = last_dts[stream->index];
   } else {
-    if ((last_dts[stream->index] != AV_NOPTS_VALUE) and (pkt->dts < last_dts[stream->index])) {
-      Warning("non increasing dts, fixing. our dts %" PRId64 " stream %d last_dts %" PRId64,
-          pkt->dts, stream->index, last_dts[stream->index]);
-      pkt->dts = last_dts[stream->index];
+    if (last_dts[stream->index] != AV_NOPTS_VALUE) {
+      if (pkt->dts < last_dts[stream->index]) {
+        Warning("non increasing dts, fixing. our dts %" PRId64 " stream %d last_dts %" PRId64 " difference %" PRId64 " last_duration %" PRId64 ". reorder_queue_size=%zu",
+            pkt->dts, stream->index, last_dts[stream->index], (last_dts[stream->index]-pkt->dts), last_duration[stream->index], reorder_queue_size);
+        pkt->dts = last_dts[stream->index]+last_duration[stream->index];
+        if (pkt->dts > pkt->pts) pkt->pts = pkt->dts; // Do it here to avoid warning below
+      } else if (pkt->dts == last_dts[stream->index]) {
+        // Commonly seen
+        Warning( "non increasing dts, fixing. our dts %" PRId64 " stream %d last_dts %" PRId64 ". reorder_queue_size=%zu",
+            pkt->dts, stream->index, last_dts[stream->index], reorder_queue_size);
+        // dts MUST monotonically increase, so add 1 which should be a small enough time difference to not matter.
+        pkt->dts = last_dts[stream->index]+last_duration[stream->index]-1;
+        if (pkt->dts > pkt->pts) pkt->pts = pkt->dts; // Do it here to avoid warning below
+      }
     }
     next_dts[stream->index] = pkt->dts + pkt->duration;
     last_dts[stream->index] = pkt->dts;
+    last_duration[stream->index] = pkt->duration;
   }
 
   if (pkt->pts == AV_NOPTS_VALUE) {
@@ -1420,8 +1574,8 @@ int VideoStore::write_packet(AVPacket *pkt, AVStream *stream) {
     pkt->pts = pkt->dts;
   }
 
-  ZM_DUMP_STREAM_PACKET(stream, (*pkt), "finished pkt");
-  Debug(3, "next_dts for stream %d has become %" PRId64 " last_dts %" PRId64,
+  ZM_DUMP_STREAM_PACKET(stream, pkt, "finished pkt");
+  Debug(2, "next_dts for stream %d has become %" PRId64 " last_dts %" PRId64,
       stream->index, next_dts[stream->index], last_dts[stream->index]);
 
   int ret = av_interleaved_write_frame(oc, pkt);
